@@ -35,7 +35,7 @@ from metview import plotting
 from metview.metviewpy.ipython import is_ipython_active, import_widgets
 from metview.metviewpy import utils
 
-__version__ = "1.13.0"
+__version__ = "1.13.1"
 
 
 # logging.basicConfig(level=logging.DEBUG, format="%(levelname)s - %(message)s")
@@ -467,7 +467,9 @@ def valid_date(*args, base=None, step=None, step_units=None):
 
 
 def sort(*args, **kwargs):
-    if len(args) != 0 and isinstance(args[0], Fieldset):
+    if "_cpp_implementation" in kwargs:
+        return call("sort", *args)
+    elif len(args) != 0 and isinstance(args[0], Fieldset):
         if len(args) == 1:
             return args[0].sort(**kwargs)
         else:
@@ -1110,9 +1112,9 @@ def list_from_metview(val):
 
 
 def datestring_from_metview(val):
-
     mdate = string_from_ffi(lib.p_value_as_datestring(val))
     dt = datetime.datetime.strptime(mdate, "%Y-%m-%dT%H:%M:%S")
+    lib.p_destroy_value(val)
     return dt
 
 
@@ -1150,7 +1152,15 @@ def handle_error(val):
 
 
 def string_from_metview(val):
-    return string_from_ffi(lib.p_value_as_string(val))
+    s = string_from_ffi(lib.p_value_as_string(val))
+    lib.p_destroy_value(val)
+    return s
+
+
+def number_from_metview(val):
+    n = lib.p_value_as_number(val)
+    lib.p_destroy_value(val)
+    return n
 
 
 class MvRetVal(Enum):
@@ -1178,7 +1188,7 @@ class ValueReturner:
 
     def __init__(self):
         self.funcs = {}
-        self.funcs[MvRetVal.tnumber.value] = lambda val: lib.p_value_as_number(val)
+        self.funcs[MvRetVal.tnumber.value] = lambda val: number_from_metview(val)
         self.funcs[MvRetVal.tstring.value] = lambda val: string_from_metview(val)
         self.funcs[MvRetVal.tgrib.value] = lambda val: Fieldset(val)
         self.funcs[MvRetVal.trequest.value] = lambda val: Request(val)
