@@ -59,8 +59,27 @@ class ParamInfo:
     NUM_RE = re.compile(r"[0-9]+")
     SURF_RE = re.compile(r"^\d+\w+")
     # SURF_NAME_MAPPER = {"t2": "2t", "q2": "2q", "u10": "10u", "v10": "10v"}
-    KNOWN_SURF_NAMES = ["2t", "2q", "10u", "10v", "msl", "wind10m"]
-    VECTOR_NAMES = ["wind10m", "wind3d", "wind"]  # the longest ones first
+    KNOWN_SURF_NAMES = [
+        "2t",
+        "2q",
+        "10u",
+        "10v",
+        "100u",
+        "100v",
+        "200u",
+        "200v",
+        "msl",
+        "wind10m",
+        "wind100m",
+        "wind200m",
+    ]
+    VECTOR_NAMES = [
+        "wind100m",
+        "wind200m",
+        "wind10m",
+        "wind3d",
+        "wind",
+    ]  # the longest ones first
 
     def __init__(self, name, meta=None, scalar=None):
         self.name = name
@@ -261,11 +280,12 @@ class ParamDesc:
             df = pd.DataFrame(md)
             md.pop("typeOfLevel")
             md.pop("level")
+
             for md_key in list(md.keys()):
-                d = df[md_key].unique().tolist()
+                d = list(df[md_key].unique())
                 self.md[md_key] = d
 
-            lev_types = df["typeOfLevel"].unique().tolist()
+            lev_types = list(df["typeOfLevel"].unique())
             for t in lev_types:
                 # print(f" t={t}")
                 self.levels[t] = []
@@ -273,7 +293,13 @@ class ParamDesc:
                 # print(q)
                 dft = df.query(q)
                 if dft is not None:
-                    self.levels[t] = dft["level"].unique().tolist()
+                    self.levels[t] = list(dft["level"].unique())
+
+            for k, v in self.md.items():
+                self.md[k] = sorted(v)
+
+            for k, v in self.levels.items():
+                self.levels[k] = sorted(v)
 
     @property
     def short_name(self):
