@@ -14,6 +14,7 @@ import glob
 import pickle
 import pytest
 import pandas as pd
+from setuptools import find_namespace_packages
 import xarray as xr
 
 import metview as mv
@@ -57,6 +58,18 @@ def get_test_data(filename):
     return f_path
 
 
+def field_equal(f1, f2, **kwargs):
+    v1 = f1.values()
+    v2 = f2.values()
+    np.testing.assert_allclose(v1, v2, **kwargs)
+
+
+def fieldset_equal(fs1, fs2, rtol=1e-7):
+    assert len(fs1) == len(fs2)
+    for i in range(len(fs1)):
+        field_equal(fs1[i], fs2[i], rtol=rtol, err_msg=f"i={i}")
+
+
 def test_version_info():
     out = mv.version_info()
     assert "metview_version" in out
@@ -1190,6 +1203,225 @@ def test_fieldset_pickling():
     os.remove(pickled_fname)
 
 
+def test_fieldset_basic_mean():
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    m = mv.mean(alldata)  # as function
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 8619.0555)
+    assert np.isclose(m.values()[2], 8588.4003)
+    m = alldata.mean()  # as method
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 8619.0555)
+    assert np.isclose(m.values()[2], 8588.4003)
+
+
+def test_fieldset_basic_mean_with_missing_vals():
+    # replace first field with all missing values
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    f1vals = alldata[0].values()
+    f1vals[:] = np.NaN
+    alldata[0] = alldata[0].set_values(f1vals)
+    m = mv.mean(alldata)  # function
+    assert len(m) == 1
+    assert np.isnan(m.values()[0])
+    assert np.isnan(m.values()[2])
+    m = mv.mean(alldata, missing=True)  # function
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 8644.7534)
+    assert np.isclose(m.values()[2], 8614.7797)
+    m = alldata.mean()  # method
+    assert len(m) == 1
+    assert np.isnan(m.values()[0])
+    assert np.isnan(m.values()[2])
+    m = alldata.mean(missing=True)  # method
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 8644.7534)
+    assert np.isclose(m.values()[2], 8614.7797)
+
+
+def test_fieldset_mean_over_dim_number():
+    # compute and check ensemble means
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    num_ens_members = len(mv.unique(alldata.grib_get_long("number")))
+    assert num_ens_members == 6
+    ens_mean = alldata.mean(dim="number")
+    # check general structure of the result
+    assert len(ens_mean) == len(alldata) / num_ens_members
+    assert mv.unique(ens_mean.grib_get_long("level")) == mv.unique(
+        alldata.grib_get_long("level")
+    )
+    assert mv.unique(ens_mean.grib_get_long("step")) == mv.unique(
+        alldata.grib_get_long("step")
+    )
+    assert mv.unique(ens_mean.grib_get_string("shortName")) == mv.unique(
+        alldata.grib_get_string("shortName")
+    )
+    assert len(mv.unique(ens_mean.grib_get_long("number"))) == 1
+    # check values for specific means #1
+    mean1_computed = ens_mean.select(shortName="z", level=1000, step=3)
+    mean1_verified = alldata.select(shortName="z", level=1000, step=3).mean()
+    assert np.array_equal(mean1_computed.values(), mean1_verified.values())
+    assert np.isclose(mean1_computed.values()[0], 1233.7)  # via calculator
+    # check values for specific means #2
+    mean2_computed = ens_mean.select(shortName="t", level=700, step=9)
+    mean2_verified = alldata.select(shortName="t", level=700, step=9).mean()
+    assert np.array_equal(mean2_computed.values(), mean2_verified.values())
+    assert np.isclose(mean2_computed.values()[2], 276.208)  # via calculator
+    # check values for specific means #3
+    mean3_computed = ens_mean.select(shortName="u", level=500, step=6)
+    mean3_verified = alldata.select(shortName="u", level=500, step=6).mean()
+    assert np.array_equal(mean3_computed.values(), mean3_verified.values())
+
+
+def test_fieldset_mean_over_dim_step():
+    # compute and check means over steps
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    num_steps = len(mv.unique(alldata.grib_get_long("step")))
+    assert num_steps == 4
+    step_mean = alldata.mean(dim="step")
+    # check general structure of the result
+    assert len(step_mean) == len(alldata) / num_steps
+    assert mv.unique(step_mean.grib_get_long("level")) == mv.unique(
+        alldata.grib_get_long("level")
+    )
+    assert mv.unique(step_mean.grib_get_long("number")) == mv.unique(
+        alldata.grib_get_long("number")
+    )
+    assert mv.unique(step_mean.grib_get_string("shortName")) == mv.unique(
+        alldata.grib_get_string("shortName")
+    )
+    assert len(mv.unique(step_mean.grib_get_long("step"))) == 1
+    # check values for specific means #1
+    mean1_computed = step_mean.select(shortName="z", level=1000, number=3)
+    mean1_verified = alldata.select(shortName="z", level=1000, number=3).mean()
+    assert np.array_equal(mean1_computed.values(), mean1_verified.values())
+    # check values for specific means #2
+    mean2_computed = step_mean.select(shortName="t", level=850, number=5)
+    mean2_verified = alldata.select(shortName="t", level=850, number=5).mean()
+    assert np.array_equal(mean2_computed.values(), mean2_verified.values())
+    assert np.isclose(mean2_computed.values()[1], 285.035)  # via calculator
+    # check values for specific means #3
+    mean3_computed = step_mean.select(shortName="u", level=500, number=1)
+    mean3_verified = alldata.select(shortName="u", level=500, number=1).mean()
+    assert np.array_equal(mean3_computed.values(), mean3_verified.values())
+
+
+def test_fieldset_mean_over_dim_step_with_expver():
+    # compute and check step means when we have a dimension that's not
+    # in our default list
+    alldata = mv.read(file_in_testdir("zt_multi_expvers.grib"))
+    num_steps = len(mv.unique(alldata.grib_get_long("step")))
+    assert num_steps == 4
+    step_mean = alldata.mean(
+        dim="step", preserve_dims=["shortName", "experimentVersionNumber"]
+    )
+    # check general structure of the result
+    assert len(step_mean) == len(alldata) / num_steps
+    assert mv.unique(step_mean.grib_get_long("level")) == mv.unique(
+        alldata.grib_get_long("level")
+    )
+    assert mv.unique(step_mean.grib_get_long("number")) == mv.unique(
+        alldata.grib_get_long("number")
+    )
+    assert mv.unique(step_mean.grib_get_string("shortName")) == mv.unique(
+        alldata.grib_get_string("shortName")
+    )
+    assert len(mv.unique(step_mean.grib_get_long("step"))) == 1
+    # check values for specific means #1
+    mean1_computed = step_mean.select(shortName="z", experimentVersionNumber="0001")
+    mean1_verified = alldata.select(
+        shortName="z", experimentVersionNumber="0001"
+    ).mean()
+    assert np.array_equal(mean1_computed.values(), mean1_verified.values())
+    # check values for specific means #2
+    mean2_computed = step_mean.select(shortName="t", experimentVersionNumber="xxyz")
+    mean2_verified = alldata.select(
+        shortName="t", experimentVersionNumber="xxyz"
+    ).mean()
+    assert np.array_equal(mean2_computed.values(), mean2_verified.values())
+    assert np.isclose(mean2_computed.values()[18], 312.2153)
+
+
+def test_fieldset_mean_over_dim_with_empty_fieldset():
+    alldata = mv.read(file_in_testdir("zt_multi_expvers.grib"))
+    with pytest.raises(ValueError):
+        alldata.select(shortName="t", level=850, number=5).mean()
+
+
+def test_fieldset_basic_sum():
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    m = mv.sum(alldata)  # as function
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 2482287.9922)
+    assert np.isclose(m.values()[2], 2473459.2813)
+    m = alldata.sum()  # as method
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 2482287.9922)
+    assert np.isclose(m.values()[2], 2473459.2813)
+
+
+def test_fieldset_basic_sum_with_missing_vals():
+    # replace first field with all missing values
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    f1vals = alldata[0].values()
+    f1vals[:] = np.NaN
+    alldata[0] = alldata[0].set_values(f1vals)
+    m = mv.sum(alldata)  # function
+    print(m.values()[0], m.values()[2])
+    assert len(m) == 1
+    assert np.isnan(m.values()[0])
+    assert np.isnan(m.values()[2])
+    m = mv.sum(alldata, missing=True)  # function
+    print(m.values()[0], m.values()[2])
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 2481044.2422)
+    assert np.isclose(m.values()[2], 2472441.8047)
+    m = alldata.sum()  # method
+    print(m.values()[0], m.values()[2])
+    assert len(m) == 1
+    assert np.isnan(m.values()[0])
+    assert np.isnan(m.values()[2])
+    m = alldata.sum(missing=True)  # method
+    print(m.values()[0], m.values()[2])
+    assert len(m) == 1
+    assert np.isclose(m.values()[0], 2481044.2422)
+    assert np.isclose(m.values()[2], 2472441.8047)
+
+
+def test_fieldset_sum_over_dim_number():
+    # compute and check ensemble sums
+    alldata = mv.read(file_in_testdir("ztu_multi_dim.grib"))
+    num_ens_members = len(mv.unique(alldata.grib_get_long("number")))
+    assert num_ens_members == 6
+    ens_sum = alldata.sum(dim="number")
+    # check general structure of the result
+    assert len(ens_sum) == len(alldata) / num_ens_members
+    assert mv.unique(ens_sum.grib_get_long("level")) == mv.unique(
+        alldata.grib_get_long("level")
+    )
+    assert mv.unique(ens_sum.grib_get_long("step")) == mv.unique(
+        alldata.grib_get_long("step")
+    )
+    assert mv.unique(ens_sum.grib_get_string("shortName")) == mv.unique(
+        alldata.grib_get_string("shortName")
+    )
+    assert len(mv.unique(ens_sum.grib_get_long("number"))) == 1
+    # check values for specific sums #1
+    sum1_computed = ens_sum.select(shortName="z", level=1000, step=3)
+    sum1_verified = alldata.select(shortName="z", level=1000, step=3).sum()
+    assert np.array_equal(sum1_computed.values(), sum1_verified.values())
+    # assert np.isclose(sum1_computed.values()[0], 1233.7)  # via calculator
+    # check values for specific sums #2
+    sum2_computed = ens_sum.select(shortName="t", level=700, step=9)
+    sum2_verified = alldata.select(shortName="t", level=700, step=9).sum()
+    assert np.array_equal(sum2_computed.values(), sum2_verified.values())
+    # assert np.isclose(sum2_computed.values()[2], 276.208)  # via calculator
+    # check values for specific sums #3
+    sum3_computed = ens_sum.select(shortName="u", level=500, step=6)
+    sum3_verified = alldata.select(shortName="u", level=500, step=6).sum()
+    assert np.array_equal(sum3_computed.values(), sum3_verified.values())
+
+
 def test_read_bufr():
     bufr = mv.read(file_in_testdir("obs_3day.bufr"))
     assert mv.type(bufr) == "observations"
@@ -2445,6 +2677,8 @@ def test_download():
 
 def test_download_gallery_data():
     fname = "z_for_spectra.grib"
+    if os.path.exists(fname):
+        os.remove(fname)
     assert not os.path.isfile(fname)
     g = mv.gallery.load_dataset(fname)
     assert mv.type(g) == "fieldset"
@@ -2455,7 +2689,11 @@ def test_download_gallery_data():
 def test_download_gallery_zipped_data():
     fname = "major_basins_of_the_world_0_0_0.zip"
     subname = "Major_Basins_of_the_World.prj"
+    if os.path.exists(fname):
+        os.remove(fname)
     assert not os.path.isfile(fname)
+    if os.path.exists(subname):
+        os.remove(subname)
     assert not os.path.isfile(subname)
     g = mv.gallery.load_dataset(fname)
     assert os.path.isfile(fname)
@@ -2546,3 +2784,66 @@ def test_mvl_ml2hpa():
     lev_divisors = [1 if x == "isobaricInhPa" else 100 for x in lev_types]
     pl_in_hpa = [a / b for a, b in zip(pls, lev_divisors)]
     assert pl_in_hpa == desired_pls
+
+
+def test_smooth_n_point():
+
+    f = mv.Fieldset(path=get_test_data("regular_ll_iPos_jNeg_iCons_multi.grib"))
+    f_ref = mv.Fieldset(path=get_test_data("ref_smooth_n_point.grib"))
+
+    meta_ref = f.grib_get_long("generatingProcessIdentifier")
+
+    r = f.smooth_n_point(n=5)
+    fieldset_equal(r, f_ref[:2])
+    meta = r.grib_get_long("generatingProcessIdentifier")
+    np.testing.assert_allclose(meta, meta_ref)
+
+    r = f.smooth_n_point(n=5, repeat=2)
+    fieldset_equal(r, f_ref[2:4])
+    meta = r.grib_get_long("generatingProcessIdentifier")
+    np.testing.assert_allclose(meta, meta_ref)
+
+    r = f.smooth_n_point(n=9, repeat=2, mode="nearest")
+    fieldset_equal(r, f_ref[4:6])
+    meta = r.grib_get_long("generatingProcessIdentifier")
+    np.testing.assert_allclose(meta, meta_ref)
+
+
+def test_smooth_gaussian():
+
+    f = mv.Fieldset(path=get_test_data("regular_ll_iPos_jNeg_iCons_multi.grib"))
+    f_ref = mv.Fieldset(path=get_test_data("ref_smooth_gaussian.grib"))
+
+    meta_ref = f.grib_get_long("generatingProcessIdentifier")
+
+    r = f.smooth_gaussian(sigma=1)
+    fieldset_equal(r, f_ref[:2])
+    meta = r.grib_get_long("generatingProcessIdentifier")
+    np.testing.assert_allclose(meta, meta_ref)
+
+    r = f.smooth_gaussian(sigma=2, repeat=2, mode="nearest")
+    fieldset_equal(r, f_ref[2:4])
+    meta = r.grib_get_long("generatingProcessIdentifier")
+    np.testing.assert_allclose(meta, meta_ref)
+
+
+def test_convolve():
+
+    f = mv.Fieldset(path=get_test_data("regular_ll_iPos_jNeg_iCons_multi.grib"))
+    f_ref = mv.Fieldset(path=get_test_data("ref_convolve.grib"))
+
+    meta_ref = f.grib_get_long("generatingProcessIdentifier")
+
+    weights = np.array(
+        [[0.0625, 0.125, 0.0625], [0.125, 0.25, 0.125], [0.0625, 0.125, 0.0625]]
+    )
+
+    r = f.convolve(weights)
+    fieldset_equal(r, f_ref[:2])
+    meta = r.grib_get_long("generatingProcessIdentifier")
+    np.testing.assert_allclose(meta, meta_ref)
+
+    r = f.convolve(weights, repeat=2, mode="nearest")
+    fieldset_equal(r, f_ref[2:4])
+    meta = r.grib_get_long("generatingProcessIdentifier")
+    np.testing.assert_allclose(meta, meta_ref)
